#' pred_RFreg_lu2019_prime

#' pred_RFreg_lu2019_prime
#' Adaptation of method from L2019 with Ranger
#' See predict_rfci_lu2019() for more details
#' @param lu2019 lu2019 model only
#' @param dataset dataset with target and predictors
#' @param target column name of the target
#' @param predictors column name of the predictors
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_lu2019_prime()
#' }
#
pred_RFreg_lu2019_prime <- function(lu2019,dataset,target,predictors){

  tmp <- predict_rfci_lu2019_prime(lu2019=lu2019,
                      X.test=as.data.frame(dataset)[,predictors],
                      Y.test = as.data.frame(dataset)[,target],
                      alpha = 0.05,
                      n.cores = 1)

  res <- data.frame(obs=tmp$obs,
                    prediction=tmp$pred,
                    lower_pred_0_05 = tmp$'lower_pred_0.05',
                    upper_pred_0_05 = tmp$'upper_pred_0.05',
                    reu_lower_pred_0_05 = tmp$'rel_lower_pred_0.05',
                    reu_upper_pred_0_05 = tmp$'rel_upper_pred_0.05',
                    perc_obs        = tmp$p_value_obs,
                    method          = rep("ranger_lu2019",length(tmp$obs)))

  return(res)
}

#' pred_RFreg_lu2019

#' pred_RFreg_lu2019
#' Adaptation of method from L2019 with Ranger
#' See predict_rfci_lu2019() for more details
#' @param lu2019 lu2019 model only
#' @param dataset validation dataset with predictors
#' @param predictors column name of the predictors
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_lu2019()
#' }
#
pred_RFreg_lu2019 <- function(lu2019,dataset,predictors){

  tmp <- predict_rfci_lu2019(lu2019=lu2019,
                      X.test=as.data.frame(dataset)[,predictors],
                      alpha = 0.05,
                      n.cores = 1)

  res <- data.frame(prediction=tmp$pred,
                    lower_pred_0_05 = tmp$'lower_pred_0.05',
                    upper_pred_0_05 = tmp$'upper_pred_0.05',
                    reu_lower_pred_0_05 = tmp$'rel_lower_pred_0.05',
                    reu_upper_pred_0_05 = tmp$'rel_upper_pred_0.05',
                    method          = rep("ranger_lu2019",length(tmp$obs)))

  return(res)
}

#' pred_RFreg_eaamm2010_prime

#' pred_RFreg_eaamm2010_prime
#' Adaptation of method from EAA2010 with Ranger
#' Method from Guide to the demonstration of equivalence of ambient air monitoring methods, expression (7.30) to (7.32)
#' https://ec.europa.eu/environment/air/quality/legislation/pdf/equivalence.pdf
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @param phi_g1 first parameter value of the output error
#' @param phi_g2 first parameter value of the output error
#' @param phi_g3 first parameter value of the output error
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_eaamm2010_prime()
#' }
#
pred_RFreg_eaamm2010_prime <- function(model_rf,dataset,target,phi_g1=5.64/1.96,phi_g2=5/100/1.96,phi_g3=112.8){

  tmp <- predict_rfci_eaam2010_prime(model_rf=model_rf,dataset=dataset,target=target,phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3)

  res <- data.frame(obs=dataset[[target]],
                    prediction=tmp$prediction,
                    se=tmp$se,
                    upper_pred_0_05 = tmp$upper_pred_0_05,
                    lower_pred_0_05 = tmp$lower_pred_0_05,
                    reu_0_05 = tmp$reu_0_05,
                    perc_obs=stats::pnorm(q=dataset[[target]], mean=tmp$prediction,sd=tmp$se),
                    method = rep("ranger_eaamm2010",length(dataset[[target]])))

  return(res)
}

#' pred_RFreg_wright2017_prime

#' pred_RFreg_wright2017_prime
#' Adaptation of method from WRIGHT2017 with Ranger
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_wright2017()
#' }
#
pred_RFreg_wright2017_prime <- function(model_rf,dataset,target){
  tmp <- predict_ranger(model_rf,dataset)
  res <- data.frame(obs=dataset[[target]],
                    prediction=tmp$predictions,
                    se=tmp$se,
                    reu_0_05 = 100*1.96*tmp$se/tmp$predictions,
                    perc_obs=stats::pnorm(q=dataset[[target]], mean=tmp$predictions,sd=tmp$se),
                    method      = rep("ranger_wright2017",length(dataset[[target]])))
  return(res)
}

#' pred_RFreg_wright2017

#' pred_RFreg_wright2017
#' Adaptation of method from WRIGHT2017 with Ranger
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_wright2017()
#' }
#
pred_RFreg_wright2017 <- function(model_rf,dataset,target){
  tmp <- predict_ranger(model_rf,dataset)
  res <- data.frame(prediction=tmp$predictions,
                    se=tmp$se,
                    reu_0_05 = 100*1.96*tmp$se/tmp$predictions,
                    method      = rep("ranger_wright2017",length(dataset[[target]])))
  return(res)
}

#' pred_RFreg_batea

#' pred_RFreg_batea_prime
#' Adaptation of BATEA with Ranger
#' @param model_batea batea model including RF ranger model
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_batea_prime()
#' }
#
pred_RFreg_batea_prime <- function(model_batea,dataset,target){

  predict_ranger_local <- function(object,dataset){
    res <- stats::predict(object=object ,tibble::as_tibble(dataset),type="response")$prediction
    return(res)
  }

  tmp <- batqa::predict_batea(model_batea,dataset,target,predict_ranger_local)

  res <- data.frame(obs = tmp$obs,
                    prediction  = tmp$prediction,
                    se_s = tmp$se_s,
                    se_o = tmp$se_o,
                    se_r = tmp$se_r,
                    se          = tmp$se_tot,
                    upper_pred_0_05 = tmp$prediction + 1.96 * tmp$se_tot,
                    lower_pred_0_05 = tmp$prediction - 1.96 * tmp$se_tot,
                    reu_0_05 = 100 * 1.96 * tmp$se_tot / tmp$prediction,
                    perc_obs    = tmp$perc_obs,
                    pvalue    = stats::pnorm(q=dataset[[target]], mean=tmp$prediction,sd=tmp$se_tot),
                    method      = tmp$method)
  return(res)
}


#' pred_RFreg_batea

#' pred_RFreg_batea
#' Adaptation of BATEA with Ranger
#' @param model_batea batea model including RF ranger model
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_RFreg_batea()
#' }
#
pred_RFreg_batea <- function(model_batea,dataset,target){

  predict_ranger_local <- function(object,dataset){
    res <- stats::predict(object=object ,tibble::as_tibble(dataset),type="response")$prediction
    return(res)
  }

  tmp <- batqa::predict_batea(model_batea,dataset,target,predict_ranger_local)

  res <- data.frame(prediction  = tmp$prediction,
                    se_s = tmp$se_s,
                    se_o = tmp$se_o,
                    se_r = tmp$se_r,
                    se          = tmp$se_tot,
                    upper_pred_0_05 = tmp$prediction + 1.96 * tmp$se_tot,
                    lower_pred_0_05 = tmp$prediction - 1.96 * tmp$se_tot,
                    reu_0_05 = 100 * 1.96 * tmp$se_tot / tmp$prediction,
                    method      = tmp$method)

  return(res)
}
