
#' predict_rfci_eaam2010_prime

#' predict_rfci_eaam2010_prime
#' Adaptation of method from EAA2010 with Ranger
#' Method from Guide to the demonstration of equivalence of ambient air monitoring methods, expression (7.30) to (7.32)
#' https://ec.europa.eu/environment/air/quality/legislation/pdf/equivalence.pdf
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @param phi_g1 first parameter value of the output error
#' @param phi_g2 first parameter value of the output error
#' @param phi_g3 first parameter value of the output error
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' predict_rfci_eaam2010_prime()
#' }
#
predict_rfci_eaam2010_prime <- function(model_rf,dataset,target,phi_g1=5.64/1.96,phi_g2=5/100/1.96,phi_g3=112.8){

  tmp <- predict_ranger(model_rf,dataset)

  # A priori output uncertainty, parameter input error from EAAMM2010
  U_g <- batqa::expr_sig_g_ts_3par(y_tilde_=dataset[[target]],phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3)

  # intercept and slope using orthogonal regression
  beta <- sor(reference=dataset[[target]],client=tmp$predictions)

  #sum of the squared residuals of the orthogonal regression
  rss_v <- rss(reference=dataset[[target]],client=tmp$predictions)

  n <- length(dataset[[target]])

  se <- sqrt( (rss_v / (n-2)) - U_g^2 + (beta[1] + (beta[2] -1) * dataset[[target]])^2  )

  res <- data.frame(obs=dataset[[target]],
                    prediction=tmp$predictions,
                    se=se,
                    upper_pred_0_05 = tmp$predictions + 1.96 * se,
                    lower_pred_0_05 = tmp$predictions - 1.96 * se,
                    reu_0_05 = 100 * 1.96 * se / tmp$predictions)
                    
  return(res)
}


#orthogonal regression
sor <- function(reference,client){
  # Use principal component analysis
  v <- stats::prcomp(cbind(reference,client))$rotation
  # Slope from the PCA output
  slope <- v[2,1] / v[1,1]
  # Intercept
  intercept <- mean(client, na.rm=TRUE) - slope * mean(reference, na.rm=TRUE)
  res <- cbind(intercept, slope)
  return(res)
}

#sum of the squared residuals of the orthogonal regression
rss <-function(reference,client){
  beta <- sor(reference,client)
  res <- sum( (client - beta[1] - beta[2] * reference)^2 , na.rm=TRUE)
  return(res)
}
